#!/usr/bin/env node
import * as path from 'path';
import { createCatalog } from './util/createCatalog.mjs';
import { createFile } from './util/createFile.mjs';
import { downloadConfig } from './util/downloadConfig.mjs';
import { isFolderExist } from './util/isFolderExist.mjs';
import { redFile } from './util/readFile.mjs';
import { readPackageVersion } from './util/readVersionPackage.mjs';

const init = async (args) => {
    const [argSnpCatalog, argTemplate, argProjectCatalog] = args;
    const version = await readPackageVersion('./package.json');
    const projectCatalog = argProjectCatalog ? argProjectCatalog : './';
    const snpCatalog = argSnpCatalog ? `${projectCatalog}/${argSnpCatalog}` : `${projectCatalog}/./snp`;
    const template = argTemplate ? argTemplate : 'node';
    const temporaryFolder = `${projectCatalog}/temporary/`;
    return await createCatalog(temporaryFolder).then(() => {
        return { snpCatalog, template, sUpdaterVersion: version, projectCatalog, temporaryFolder };
    });
};
const createConfigFile = async (config) => {
    const { snpCatalog, template, sUpdaterVersion } = config;
    const filePath = `${snpCatalog}/snp.config.json`;
    await isFolderExist({
        folderPath: snpCatalog,
        createFolder: true,
    });
    await createFile({
        filePath,
        content: JSON.stringify({ sUpdaterVersion, template }),
    });
    return config;
};
const downloadRemoteConfig = async (config) => {
    const fileMap = await downloadConfig(config.template, config.snpCatalog, config.temporaryFolder);
    const organizeFileMap = (fileMap) => {
        const files = {};
        fileMap.forEach((file) => {
            const fileName = file.substring(0, file.lastIndexOf('-'));
            if (!files[fileName]) {
                files[fileName] = [];
            }
            files[fileName].push(`${config.snpCatalog}/${file}`);
        });
        return { fileMap, files };
    };
    return { ...config, fileMap: organizeFileMap(fileMap) };
};
const buildConfig = async (config) => {
    // console.log(config);
    const buildFile = async ({ fileMap, }) => {
        const files = fileMap.files;
        console.log({ files });
        const contentFile = Object.keys(files);
        for (const fileName of contentFile) {
            const defaultFile = files[fileName].find((element) => element.includes('-default.md')) || '';
            const contentFile = await redFile(defaultFile);
            await createFile({
                filePath: `${config.projectCatalog}/${fileName}`,
                content: contentFile,
            });
        }
    };
    buildFile(config);
};
const cleanUp = async (config) => {
    console.log(config);
    // ENDWORK HERE - CLEAN UP TEMP FILE/**/
};
let args = process.argv.slice(3);
if (Boolean(process.env.SDEBUG)) {
    // argSnpCatalog - snp catalog
    // argTemplate - template remote github
    // argProjectCatalog - project catalog
    args = [path.join('./.snp'), 'node', './test/fakeProjectRootfolder'];
}
init(args)
    .then((config) => createConfigFile(config))
    .then((config) => downloadRemoteConfig(config))
    .then((config) => buildConfig(config))
    .then((config) => cleanUp(config));

export { buildConfig, cleanUp, createConfigFile, downloadRemoteConfig, init };
